-----------------------------------------------------------------------------------------------
-- Client Lua Script for StopWatch
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- StopWatch Module Definition
-----------------------------------------------------------------------------------------------
local StopWatch = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function StopWatch:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	self.nCurrentTime = 0
	self.nLastTime = 0
	self.bRunning = false
	self.tSaved = {tTimes = {0,0,0,0,0,0,0,0,0,0},tAnchorOffsets = {-120,-20,80,20}, bVisible = true}
	self.nIndex = 0
    return o
end

function StopWatch:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- StopWatch OnLoad
-----------------------------------------------------------------------------------------------
function StopWatch:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("StopWatch.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- StopWatch OnDocLoaded
-----------------------------------------------------------------------------------------------
function StopWatch:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "StopWatchForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		self.wndTimer = Apollo.LoadForm(self.xmlDoc, "TimerForm", nil, self)
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		--Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("sw", "OnStopWatchOn", self)
		Apollo.RegisterEventHandler("SystemKeyDown", "OnSystemKeyDown", self)
		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
		Apollo.RegisterEventHandler("ToggleStopWatch", "OnStopWatchOn", self)
		--self.timer = ApolloTimer.Create(1.0, false, "OnTimer", self)
		--self.timer:Stop()

		Apollo.RegisterEventHandler("NextFrame","OnUpdate", self)
		self.wndTimer:SetText(self:FormatTime(self.nCurrentTime))

		-- Do additional Addon initialization here
		self.wndTimer:SetAnchorOffsets(unpack(self.tSaved.tAnchorOffsets))
		self.wndTimer:Show(self.tSaved.bVisible, true)
		Event_FireGenericEvent("InterfaceMenuList_AlertAddOn", "StopWatch", {self.tSaved.bVisible, nil, 0})

	end
end

-----------------------------------------------------------------------------------------------
-- StopWatch Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/sw"
function StopWatch:OnStopWatchOn()
--Print("This does nothing...yet.")
	self.tSaved.bVisible = not self.wndTimer:IsShown()
	self.wndTimer:Show(self.tSaved.bVisible, true)
	Event_FireGenericEvent("InterfaceMenuList_AlertAddOn", "StopWatch", {self.tSaved.bVisible, nil, 0})
end

function StopWatch:OnInterfaceMenuListHasLoaded()
Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "StopWatch", {"ToggleStopWatch", "", "CRB_Basekit:kitIcon_Holo_Clock"})
Event_FireGenericEvent("InterfaceMenuList_AlertAddOn", "StopWatch", {self.tSaved.bVisible, nil, 0})

--self:UpdateInterfaceMenuAlerts()
end

-- on timer
--function StopWatch:OnTimer()
	
--end

function StopWatch:OnSave(eLevel)
    if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
        return nil
    end
	self.tSaved.tAnchorOffsets = {self.wndTimer:GetAnchorOffsets()}
	return self.tSaved
end

function StopWatch:OnRestore(eLevel,tData)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
        return nil
    end
	for key, value in pairs(self.tSaved) do
		if tData[key] == nil then
			tData[key] = value
		end
	end
    self.tSaved = tData
	if self.wndTimer then
	self.wndTimer:SetAnchorOffsets(unpack(self.tSaved.tAnchorOffsets))
	self.wndTimer:Show(self.tSaved.bVisible, true)
	Event_FireGenericEvent("InterfaceMenuList_AlertAddOn", "StopWatch", {self.tSaved.bVisible, nil, 0})
	end
end

--function StopWatch:OnKeyDown(iKey)
--Print("Normal:"..iKey)
--end

function StopWatch:EditTimer(nChange)
	if self.bRunning then
		self.nDifference = self.nDifference + nChange
	else
		self.nCurrentTime = self.nCurrentTime + nChange
		self.wndTimer:SetText(self:FormatTime(self.nCurrentTime))
	end
end



function StopWatch:OnSystemKeyDown(iKey)
if not self.tSaved.bVisible then return end
--Print("System:"..iKey)
if self.bEditMode then
	local time = self.bRunning and self.nDifference + self.nDirection * os.clock() or self.nCurrentTime
	if iKey == 103 then
		self:EditTimer(60)
	elseif iKey == 104 then
		self:EditTimer(1)
	elseif iKey == 105 then
		self:EditTimer(0.01)
	elseif iKey == 100 then
		self:EditTimer(-1 * math.floor(time/60)*60)
	elseif iKey == 101 then
		self:EditTimer(-1 *  math.floor(time % 60))
	elseif iKey == 102 then
		self:EditTimer(-1 * (time % 1))
	elseif iKey == 97 then
		if time - 60 < 0 then
			self.nCurrentTime = 0
			self.wndTimer:SetText(self:FormatTime(self.nCurrentTime))
		else
			self:EditTimer(-60)
		end		
	elseif iKey == 98 then
		if time - 1 < 0 then
			self.nCurrentTime = 0
			self.wndTimer:SetText(self:FormatTime(self.nCurrentTime))
		else
			self:EditTimer(-1)
		end	
	elseif iKey == 99 then
		if time - 0.01 < 0 then
			self.nCurrentTime = 0
			self.wndTimer:SetText(self:FormatTime(self.nCurrentTime))
		else
			self:EditTimer(-0.01)
		end		
	elseif iKey == 96 then
		self.bEditMode = false
	end
else
	if iKey == 103 then
		self.nLastTime = self.nCurrentTime
		self.nCurrentTime = 0
		self.wndTimer:SetText(self:FormatTime(self.nCurrentTime))
		if self.bRunning then
			if self.nDirection == 1 then
				self:StartUp()
			else
				self:StartDown()
			end
		end
	elseif iKey == 104 then
		self.nCurrentTime = self.nLastTime
		self.wndTimer:SetText(self:FormatTime(self.nCurrentTime))
		if self.bRunning then
			if self.nDirection == 1 then
				self:StartUp()
			else
				self:StartDown()
			end
		end
	elseif iKey == 105 then
		--restore last saved
		self.nLastTime = self.nCurrentTime
		self.nCurrentTime = self.tSaved.tTimes[1]
		self.wndTimer:SetText(self:FormatTime(self.nCurrentTime))
		if self.bRunning then
			if self.nDirection == 1 then
				self:StartUp()
			else
				self:StartDown()
			end
		end
	elseif iKey == 100 then
		if self.bRunning then
			self:Pause()
		end
		self:StartDown()
	elseif iKey == 101 then
		if self.bRunning then
			self:Pause()
		elseif self.nDirection == 1 then
			self:StartUp()
		else
			self:StartDown()
		end
	elseif iKey == 102 then
		if self.bRunning then
			self:Pause()
		end
		self:StartUp()
	elseif iKey == 96 then --97 still open
		--set time on 0 key
		self.bEditMode = true
	elseif iKey == 98 then
		self:TriggerLoopMode()
	elseif iKey == 99 then
		local time = self.bRunning and self.nDifference + self.nDirection * os.clock() or self.nCurrentTime
		table.insert(self.tSaved.tTimes, 1, time)
		self.tSaved.tTimes[11] = nil
	end
end	
end

function StopWatch:OnUpdate()
	if not self.bRunning then return end
	local time = self.nDifference + self.nDirection * os.clock()
	self.wndTimer:SetText(self:FormatTime(time))
	if time < 0 then
		--ring
		self:Alarm()
	end
end

function StopWatch:FormatTime(nTime)
	local minutes = math.floor(nTime/60)
	local rest = nTime % 60
	return string.format("%.2d",minutes)..":"..string.format("%05.2f",rest)
end

function StopWatch:StartUp()
	self.bRunning = true
	self.nLastTime = self.nCurrentTime
	self.nDifference = self.nCurrentTime - os.clock()
	self.nDirection = 1
	self.nIndex = 0
end

function StopWatch:StartDown()
	if self.nCurrentTime == 0 then
		self:Alarm() return
	end
	self.bRunning = true
	self.nLastTime = self.nCurrentTime
	self.nDifference = self.nCurrentTime + os.clock()
	self.nDirection = -1
	self.nIndex = 0
end

function StopWatch:Pause()
	self.bRunning = false
	self.nCurrentTime = self.nDifference + self.nDirection * os.clock()
	self.wndTimer:SetText(self:FormatTime(self.nCurrentTime))
end

function StopWatch:TriggerLoopMode()
	self.bLoopMode = not self.bLoopMode
	Print("StopWatch: Loop Mode "..(self.bLoopMode and "activated." or "deactivated."))
end

function StopWatch:Alarm()
	self.bRunning = false
	self.nCurrentTime = 0
	self.wndTimer:SetText(self:FormatTime(self.nCurrentTime))
	Sound.Play(212)
	if self.bLoopMode then
		self.nCurrentTime = self.nLastTime
		self:StartDown()
	end
end

-----------------------------------------------------------------------------------------------
-- StopWatchForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function StopWatch:OnOK()
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function StopWatch:OnCancel()
	self.wndMain:Close() -- hide the window
end


---------------------------------------------------------------------------------------------------
-- TimerForm Functions
---------------------------------------------------------------------------------------------------

function StopWatch:OnMouseWheel( wndHandler, wndControl, nLastRelativeMouseX, nLastRelativeMouseY, fScrollAmount, bConsumeMouseWheel )
--Print(fScrollAmount)
	if self.bRunning then return end
	if self.nIndex == 0 then
		self.tSaved.tTimes[0] = self.nCurrentTime
	end
	self.nIndex = self.nIndex + fScrollAmount
	if self.nIndex <= 0 then
		self.nIndex = 0 
		--self.wndTimer:SetText(self:FormatTime(self.nCurrentTime))
		--return
	end
	if self.nIndex > 10 then
		self.nIndex = 10
	end
	--self.nLastTime = self.nCurrentTime
	self.nCurrentTime = self.tSaved.tTimes[self.nIndex]
	self.wndTimer:SetText(self:FormatTime(self.nCurrentTime))
end

-----------------------------------------------------------------------------------------------
-- StopWatch Instance
-----------------------------------------------------------------------------------------------
local StopWatchInst = StopWatch:new()
StopWatchInst:Init()
